module Spellcheck where

import Prelude hiding (Word)
import System.Environment
import Control.Exception
import System.IO(hFlush, stdout)

import Dictionary
import Words 

prompt :: String-> IO String
prompt p = do putStr p; hFlush stdout; getLine

resolve :: Word-> Dictionary-> IO (Word, Dictionary)
resolve w ds = do
  putStrLn $ "Unknown word: "++ w
  x<- prompt "[1] Add to dictionary, [2] Ignore, [3] Correct ? "
  case x of "1" -> return (w, addSpelling ds w) 
            "2" -> return (w, addIgnored ds w)
            "3" -> do nu<- prompt "Enter correction: "
                      if contains ds nu 
                        then return (nu, ds)
                        else resolve nu ds
            x -> do putStrLn "I beg your pardon?"; resolve w ds

          
checkChunk :: Chunk-> Dictionary -> IO (Chunk, Dictionary)
checkChunk (Word w) ds | not (contains ds w) = do (w, d)<- resolve w ds; return (Word w, d)
checkChunk c ds = return (c, ds)

lift2M :: (a-> b-> IO (a, b))-> [a]-> b-> IO ([a], b)
lift2M act as b = lift2M' as b [] where
  lift2M' [] b res = return (reverse res, b)
  lift2M' (a:as) b res = do
    (a1, b1)<- act a b
    lift2M' as b1 (a1:res) 

processDoc :: FilePath-> FilePath-> IO ()
processDoc inp outp = catch (do
  content <- readFile inp
  d <- readDictionary "dictionary.txt"
  (corrected, d') <- lift2M checkChunk (chunks content) d
  writeFile outp (unchunk corrected)
  writeDictionary d' "dictionary.txt") 
  (\e-> putStrLn $ "ERROR: "++ show (e::IOError))

main :: IO ()
main = do
  inp:outp:_ <- getArgs 
  processDoc inp outp